<?xml version="1.0" encoding='iso-8859-1'?>

<!-- ====================================================================== 

     ====================================================================== -->

<xsl:stylesheet xmlns:rdf    = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'
		xmlns:xsl    = "http://www.w3.org/1999/XSL/Transform"
		xmlns:dc     = "http://purl.org/dc/elements/1.1/"
		xmlns:svg    = "http://www.w3.org/2000/svg"
		xmlns:xlink  = "http://www.w3.org/1999/xlink"
		xmlns:doc    = "http://xsltsl.org/xsl/documentation/1.0"
                xmlns:date   = "http://exslt.org/dates-and-times"
		xmlns:admin  = "http://webns.net/mvcb/"
		xmlns:markup = "http://xsltsl.org/markup"

		extension-element-prefixes="date"
		exclude-result-prefixes="doc date markup"
		version = "1.0" >

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:import href = "fxsl/exp.xsl" />

<!-- ====================================================================== 

     ====================================================================== -->

     <doc:reference>
      <referenceinfo>

       <title>svg-travel-map</title>

       <abstract>
        <para>This is an <acronym>XSLT</acronym> 1.0 stylesheet for transforming an <ulink url = "http://www.bryanboyer.com/indyjunior/">IndyJunior</ulink> <acronym>XML</acronym> document
        into an <ulink url =
        "http://aaronland.info/xsl/svg/svg-travel-map/example.svg">animated <acronym>SVG</acronym> travel map</ulink>.</para>

       </abstract>

       <author>
	<surname>Cope</surname>
	<firstname>Aaron</firstname>
	<othername>Straup</othername>
	<email>asc@vineyard.net</email>
       </author>

       <legalnotice>
        <para>This work is licensed under the Creative Commons
        Attribution-ShareAlike License. To view a copy of this
        license, visit <ulink url = "http://creativecommons.org/licenses/by-sa/1.0/">http://creativecommons.org/licenses/by-sa/1.0/</ulink> or send a letter to Creative Commons, 559 Nathan Abbott Way,
        Stanford, California 94305, USA.</para> 
       </legalnotice>

       <revhistory>
        <revision>
	 <revnumber>1.0</revnumber>
	 <date>2004-09-15</date>
	 <revremark>
	  <para>Initial release.</para>
         </revremark>
	</revision>
       </revhistory>

      </referenceinfo>

      <refentry>
       <refsection>
        <title>Usage</title>

	<example>
	 <programlisting><![CDATA[
 $> xsltproc --stringparam css_href 'default.css' --stringparam map_href 'world-1024.png' \
             svg-travel-map.xsl locations.xml > example.svg
	 ]]></programlisting>
	</example>
	 
	<para>Consult the documentation for your favourite
       <acronym>XSLT</acronym> processor. This template accepts the
       following parameters : </para>

	<itemizedlist>

	 <listitem>
	  <formalpara>
	   <title>title</title>
	   <para>String. Sets the document's title in the metadata section, using the Dublin Core namespace.</para>
	  </formalpara> 
         </listitem>

	 <listitem>
	  <formalpara>
	   <title>description</title>
	   <para>String. Sets the document's description in the metadata section, using the Dublin Core namespace.</para>
	  </formalpara> 
         </listitem>

	 <listitem>
	  <formalpara>
	   <title>publisher</title>
	   <para>String. Sets the document's publisher in the metadata section, using the Dublin Core namespace. If no value is defined then the <acronym>URL</acronym> of the <acronym>XSLT</acronym> processor is used.</para>
	  </formalpara> 
         </listitem>

	 <listitem>
	  <formalpara>
	   <title>language</title>
	   <para>String. Sets the document's language in the metadata section, using the Dublin Core namespace.</para>
	  </formalpara> 
         </listitem>

	 <listitem>
	  <formalpara>
	   <title>about</title>
	   <para>String. Sets the document's <acronym>URI</acronym> using the <acronym>RDF</acronym> namespace.</para>
	  </formalpara> 
         </listitem>

	 <listitem>
	  <formalpara>
	   <title>css_href</title>
	   <para>String. Sets the <acronym>URL</acronym> for the document's stylesheet.</para>
	  </formalpara> 
         </listitem>

	 <listitem>
	  <formalpara>
	   <title>travel_duration</title>
	   <para>Int. The number of seconds that a line connecting two locations will take to render. Default is 2.</para>
	  </formalpara> 
         </listitem>

	 <listitem>
	  <formalpara>
	   <title>map_href</title>
	   <para>String. Sets the <acronym>URL</acronym> for the document's map data. This package contains both a <acronym>PNG</acronym> and <acronym>SVG</acronym> map of the world, rendered at 1024 x 494 pixels.</para>
	  </formalpara> 
         </listitem>

	 <listitem>
	  <formalpara>
	   <title>map_h</title>
	   <para>Int. The height of the document's map data. Default is 494, the height of the <filename>world-1024</filename> files included with this package.</para>
	  </formalpara> 
         </listitem>

	 <listitem>
	  <formalpara>
	   <title>map_w</title>
	   <para>Int. The width of the document's map data. Default is 1024, the height of the <filename>world-1024</filename> files included with this package.</para>
	  </formalpara> 
         </listitem>

	 <listitem>
	  <formalpara>
	   <title>map_min_x</title>
	   <para>Int. The  of the document's map data. Default is -180, the FOO of the <filename>world-1024</filename> files included with this package.</para>
	  </formalpara> 
         </listitem>

	 <listitem>
	  <formalpara>
	   <title>map_max_y</title>
	   <para>Float. The FOO of the document's map data. Default is 83.674733, the FOO of the <filename>world-1024</filename> files included with this package.</para>
	  </formalpara> 
         </listitem>

	 <listitem>
	  <formalpara>
	   <title>map_scale</title>
	   <para>Float. The scale used to render the document's map data. Default is 2.84444444444444, the scale used to render the <filename>world-1024</filename> files included with this package.</para>
	  </formalpara> 
         </listitem>

	 <listitem>
	  <formalpara>
	   <title>map_decimals</title>
	   <para>Int. The number of decimal points used to render the document's map data. Default is 1, the number of decimal points used to render the <filename>world-1024</filename> files included with this package.</para>
	  </formalpara> 
         </listitem>

	</itemizedlist>

	 <para>You
        should be able to pass it the same <acronym>XML</acronym>
        file unaltered with two exceptions :</para>

	<itemizedlist>
	 <listitem>
	  <para>trans-Pacific travels must be indicated by a transpacific attribute.</para>
 	  <example>
	   <programlisting><![CDATA[
<location arrival="2004-11-01">
  <name>Sydney</name>
  <latitude>33S</latitude>
  <longitude>151E</longitude>
</location>
<location arrival="2005-01-01" transpacific = "true">
  <name>Honolulu</name>
  <latitude>21N</latitude>
  <longitude>157W</longitude>
</location>]]>
           </programlisting>
          </example>
	 </listitem>
	 <listitem>
	  <para>If you have two locations with the same arrival date,
	  you will need to make sure that the dates are
	  different. This is a limitation of <acronym>XSL</acronym>
	  (I think) sorting that can be easily handled by speciying
	  the date using a complete <acronym>W3C</acronym> datetime
	  string.</para>
 	  <example>
	   <programlisting><![CDATA[
<location arrival="2005-01-01" transpacific = "true">
  <name>Honolulu</name>
  <latitude>21N</latitude>
  <longitude>157W</longitude>
</location>
<location arrival="2005-01-21T15:35:00" transpacific = "true">
  <name>Hong Kong</name>
  <latitude>22N</latitude>
  <longitude>114E</longitude>
</location>]]></programlisting>
	  </example>
	 </listitem>
	</itemizedlist>

       </refsection>

       <refsection>
        <title>Requirements</title>

	<itemizedlist>
	 <listitem>
	  <para>Version 1.2 of the <ulink url = "http://fxsl.sourceforge.net/">Functional Programming Library
        for <acronym>XSLT</acronym></ulink> templates.</para>
	 </listitem>
	 <listitem>
	  <para>A <acronym>XSLT</acronym> processor with support for
	 the <ulink url = "http://www.exslt.org/date/index.html"><acronym>EXSLT</acronym> date and time functions</ulink>.</para>
	 </listitem>
	</itemizedlist>

       </refsection>

      </refentry>

     </doc:reference>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:output method   = "xml" />
     <xsl:output encoding = "UTF-8" />
     <xsl:output indent   = "yes" />

     <xsl:strip-space elements = "*" />

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:param name = "title" />
     <xsl:param name = "description" />
     <xsl:param name = "publisher" />
     <xsl:param name = "language" />
     <xsl:param name = "about" />

     <xsl:param name = "travel_duration" select = "2" />

     <xsl:param name = "css_href" />
     <xsl:param name = "map_href" />

     <xsl:param name = "map_h"        select = "494" />
     <xsl:param name = "map_w"        select = "1024" />

     <xsl:param name = "map_min_x"    select = "-180" />
     <xsl:param name = "map_max_y"    select = "83.674733" />

     <xsl:param name = "map_scale"    select = "2.84444444444444" />
     <xsl:param name = "map_decimals" select = "1" />

     <xsl:variable name = "_exp">
      <xsl:call-template name = "pow">
       <xsl:with-param name = "pBase"  select = "10" />
       <xsl:with-param name = "pPower" select = "$map_decimals" />
      </xsl:call-template>
     </xsl:variable>

     <xsl:variable name = "map_exp" select = "round($_exp)" />

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template match = "/">

      <xsl:call-template name = "add-styles" />

      <svg>
       <xsl:attribute name = "height">
        <xsl:value-of select = "$map_h" />
       </xsl:attribute>
       <xsl:attribute name = "width">
        <xsl:value-of select = "$map_w" />
       </xsl:attribute>

       <xsl:call-template name = "add-scripts" />
       <xsl:call-template name = "add-metadata" />
       <xsl:call-template name = "add-defs" />
       <xsl:call-template name = "add-map" />

       <xsl:call-template name = "add-arrow" />
       <xsl:call-template name = "add-locations" />

      </svg>

     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "add-styles">
      <xsl:if test = "$css_href">
       <xsl:processing-instruction name="xml-stylesheet">
        <xsl:text>type="text/css"</xsl:text>
        <xsl:text>href="</xsl:text>
        <xsl:value-of select="$css_href"/>
        <xsl:text>"</xsl:text>
       </xsl:processing-instruction>
      </xsl:if>
     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "add-scripts">
      <script>
       <xsl:attribute name = "type">
        <xsl:text>text/ecmascript</xsl:text>
       </xsl:attribute>

      <xsl:call-template name = "markup:cdata-section">
       <xsl:with-param name = "text">

function show_place (evt,t_id) {
	 txt = svgDocument.getElementById(t_id);
	 txt.setAttribute("style","display:block;");
}

function hide_place (evt,t_id) {
	 txt = svgDocument.getElementById(t_id);
	 txt.setAttribute("style","display:none;");
}

       </xsl:with-param>
      </xsl:call-template>

      </script>

     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "add-metadata">
      <metadata>
       <rdf:RDF>
        <rdf:Description>

	 <xsl:attribute name = "rdf:about">
	  <xsl:choose>
	   <xsl:when test = "$about">
	   </xsl:when>
	   <xsl:otherwise>
	    <xsl:text>#</xsl:text>
	   </xsl:otherwise>
	  </xsl:choose>
	 </xsl:attribute>

	 <xsl:if test = "$title">
	  <dc:title>
	   <xsl:value-of select = "$title" />
	  </dc:title>
	 </xsl:if>

	 <xsl:if test = "$description">
          <dc:description>
	   <xsl:value-of select = "$description" />
	  </dc:description>
	 </xsl:if>

         <dc:publisher>
	  <xsl:choose>
	   <xsl:when test = "$publisher">
	   </xsl:when>
	   <xsl:otherwise>
	    <xsl:value-of select = "system-property('xsl:vendor-url')" />
	   </xsl:otherwise>
	  </xsl:choose>
	 </dc:publisher>

         <dc:date>
	  <xsl:value-of select = "date:date-time()" />
	 </dc:date>

         <dc:format>image/svg+xml</dc:format>

	 <xsl:if test = "$language">
          <dc:language><xsl:value-of select = "$language" /></dc:language>
	 </xsl:if> 

	 <admin:generatorAgent>
	  <xsl:attribute name = "rdf:Resource">
	   <xsl:text>http://aaronland.info/xsl/svg/travel-map</xsl:text>
	  </xsl:attribute>
	 </admin:generatorAgent>	 

       </rdf:Description>
       </rdf:RDF>
      </metadata>
     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "add-defs">
      <defs>
       <g id = "you-are-here">
        <polyline>
	 <xsl:attribute name = "points">
	  <xsl:text>12.5 0, 17.5 0, 17.5 10, 22.5 10, 15 20, 7.5 10, 12.5 10, 12.5 0</xsl:text>
	 </xsl:attribute>
	</polyline>
       </g>
      </defs>
     </xsl:template>
     
<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "add-map">
      <xsl:if test = "$map_href">
      <g>
       <xsl:attribute name = "id">map</xsl:attribute>

       <image>
        <xsl:attribute name = "width">
	 <xsl:value-of select = "$map_w" />
	</xsl:attribute>
        <xsl:attribute name = "height">
	 <xsl:value-of select = "$map_h" />
	</xsl:attribute>
        <xsl:attribute name = "x">0</xsl:attribute>
        <xsl:attribute name = "y">0</xsl:attribute>
        <xsl:attribute name = "xlink:href">
	 <xsl:value-of select = "$map_href" />
	</xsl:attribute>
       </image>
      </g>
      </xsl:if>
     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "add-locations">
      <g>
       <xsl:attribute name = "id">locations</xsl:attribute>

       <xsl:for-each select = "/travels/location">
        <xsl:sort select    = "date:seconds(@arrival)"
		  data-type = "number" />

        <xsl:call-template name = "plot-location" />
       </xsl:for-each>

      </g>
     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "add-arrow">
      <xsl:variable name = "now"  select = "date:seconds(date:date-time())" />

      <xsl:for-each select = "/travels/location[date:seconds(@arrival) &lt; $now]">
       <xsl:sort select    = "date:seconds(@arrival)"
		 data-type = "number" 
		 order     = "descending" />

        <xsl:call-template name = "you-are-here">
	 <xsl:with-param name = "lat"  select = "latitude" />
	 <xsl:with-param name = "long" select = "longitude" />
	 <xsl:with-param name = "inst" select = "position()" />
	</xsl:call-template>
      </xsl:for-each>
     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "plot-location">

      <g>
      <xsl:attribute name = "class">location</xsl:attribute>

      <title><xsl:value-of select = "name" /></title>

      <xsl:choose>
       <xsl:when test = "url">
       <a>
        <xsl:attribute name = "xlink:href">
	 <xsl:value-of select = "url" />
        </xsl:attribute>
        <xsl:call-template name = "draw-location" />
       </a> 
       </xsl:when>
       <xsl:otherwise>
        <xsl:call-template name = "draw-location" />
       </xsl:otherwise>      
      </xsl:choose>

      </g>

     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "draw-location">

       <xsl:variable name = "cx">
       <xsl:call-template name = "calc-x">
        <xsl:with-param name = "long">
	 <xsl:value-of select = "longitude" />
        </xsl:with-param>
       </xsl:call-template>
       </xsl:variable>

       <xsl:variable name = "cy">
       <xsl:call-template name = "calc-y">
        <xsl:with-param name = "lat">
	 <xsl:value-of select = "latitude" />
        </xsl:with-param>
       </xsl:call-template>
       </xsl:variable>

       <xsl:variable name = "now"  select = "date:seconds(date:date-time())" />
       <xsl:variable name = "secs" select = "date:seconds(@arrival)" />

       <xsl:variable name = "plot_when">
        <xsl:choose>
	 <xsl:when test = "$secs > $now">
	  <xsl:text>future</xsl:text>
	 </xsl:when>
	 <xsl:otherwise>
	  <xsl:text>past</xsl:text>
	 </xsl:otherwise>
	</xsl:choose>
       </xsl:variable>

       <xsl:variable name = "t_id">
        <xsl:text>t_</xsl:text>
	<xsl:value-of select = "generate-id()" />
       </xsl:variable>

       <circle>

        <xsl:attribute name = "class">
	 <xsl:text>location-outer-</xsl:text>
	 <xsl:value-of select = "$plot_when" />
	</xsl:attribute>

        <xsl:attribute name = "r">0</xsl:attribute>

        <xsl:attribute name = "cx">
         <xsl:value-of select = "$cx" />
        </xsl:attribute>

        <xsl:attribute name = "cy">
	 <xsl:value-of select = "$cy" />
	</xsl:attribute>

	<xsl:attribute name = "onmouseout">
	 <xsl:text>hide_place(evt,'</xsl:text>
	 <xsl:value-of select = "$t_id" />
	 <xsl:text>')</xsl:text>
	</xsl:attribute>

	<xsl:attribute name = "onmouseover">
	 <xsl:text>show_place(evt,'</xsl:text>
	 <xsl:value-of select = "$t_id" />
	 <xsl:text>')</xsl:text>
	</xsl:attribute>

	<animate>
         <xsl:attribute name = "attributeName">r</xsl:attribute>
         <xsl:attribute name = "attributeType">XML</xsl:attribute>
	 <xsl:attribute name = "from">0</xsl:attribute>
	 <xsl:attribute name = "to">5</xsl:attribute>
	 <xsl:attribute name = "begin">
	  <xsl:value-of select = "(position() - 1) * $travel_duration" />
	  <xsl:text>s</xsl:text>
         </xsl:attribute>
	 <xsl:attribute name = "dur">2s</xsl:attribute>
	 <xsl:attribute name = "fill">freeze</xsl:attribute>
	</animate>

       </circle>

       <circle>

        <xsl:attribute name = "class">
	 <xsl:text>location-inner-</xsl:text>
	 <xsl:value-of select = "$plot_when" />
	</xsl:attribute>

        <xsl:attribute name = "r">0</xsl:attribute>

        <xsl:attribute name = "cx">
	 <xsl:value-of select = "$cx" />
	</xsl:attribute>

        <xsl:attribute name = "cy">
	 <xsl:value-of select = "$cy" />
	</xsl:attribute>

	<animate>
         <xsl:attribute name = "attributeName">r</xsl:attribute>
         <xsl:attribute name = "attributeType">XML</xsl:attribute>
	 <xsl:attribute name = "from">0</xsl:attribute>
	 <xsl:attribute name = "to">1</xsl:attribute>
	 <xsl:attribute name = "begin">
	  <xsl:value-of select = "((position() - 1) * $travel_duration) + 1" />
	  <xsl:text>s</xsl:text>
         </xsl:attribute>
	 <xsl:attribute name = "dur">1s</xsl:attribute>
	 <xsl:attribute name = "fill">freeze</xsl:attribute>
	</animate>

       </circle>

      <xsl:call-template name = "goto-next">
       <xsl:with-param name = "cx" select = "$cx" />
       <xsl:with-param name = "cy" select = "$cy" />
       <xsl:with-param name = "arrival" select = "arrival" />
       <xsl:with-param name = "now" select = "$now" />
      </xsl:call-template>

      <!-- text boxes -->

       <xsl:variable name = "font_h" select = "10" />
       <xsl:variable name = "font_w" select = "round($font_h * 0.6)" />

       <xsl:variable name = "text_indent">
        <xsl:value-of select = "round($font_h * 0.5)" />
       </xsl:variable>

       <xsl:variable name = "_text_len">
        <xsl:choose>
	 <xsl:when test = "string-length(@arrival) > string-length(name)">
	  <xsl:value-of select = "string-length(@arrival)" />
	 </xsl:when>
	 <xsl:otherwise>
	  <xsl:value-of select = "string-length(name)" />
	 </xsl:otherwise>
	</xsl:choose>
       </xsl:variable>

       <xsl:variable name = "text_length">
        <xsl:value-of select = "($_text_len * $font_w) + ($text_indent * 2)" />
       </xsl:variable>

       <xsl:variable name = "text_height">
        <xsl:value-of select = "($font_h * 2) + ($text_indent * 5)" />
       </xsl:variable>

       <xsl:variable name = "text_x">
        <xsl:choose>
	 <xsl:when test = "(($cx + 10) + $text_length) >= 1024">
 	  <xsl:value-of select = "($cx - 10) - $text_length" />
	 </xsl:when>
	 <xsl:otherwise>
	  <xsl:value-of select = "$cx + 10" />
	 </xsl:otherwise>
	</xsl:choose>
       </xsl:variable>

       <xsl:variable name = "text_y">
        <xsl:choose>
	 <xsl:when test = "(($cy - 10) - $text_height) &lt;= 0">
 	  <xsl:value-of select = "$cy + 10" />
	 </xsl:when>
	 <xsl:otherwise>
	  <xsl:value-of select = "($cy - 10) - $text_height" />
	 </xsl:otherwise>
	</xsl:choose>
       </xsl:variable>

       <g>

        <xsl:attribute name = "id">
	 <xsl:value-of select = "$t_id" />
	</xsl:attribute>

        <xsl:attribute name = "class">
	 <xsl:text>blurbs</xsl:text>
	</xsl:attribute>

        <xsl:attribute name = "style">
	 <xsl:text>display:none;</xsl:text>
	</xsl:attribute>

	<rect>
	 <xsl:attribute name = "class">
	  <xsl:text>blurbs-bg</xsl:text>
	 </xsl:attribute>
	 
	 <xsl:attribute name = "x">
	  <xsl:value-of select = "$text_x" />
	 </xsl:attribute>

	 <xsl:attribute name = "y">
	  <xsl:value-of select = "$text_y" />
	 </xsl:attribute>

	 <xsl:attribute name = "height">
	  <xsl:value-of select = "$text_height" />
	 </xsl:attribute>

	 <xsl:attribute name = "width">
	  <xsl:value-of select = "$text_length" />
	 </xsl:attribute>

	</rect>
     
       <text>
	<xsl:attribute name = "x">
	 <xsl:value-of select = "$text_x + $text_indent" />
	</xsl:attribute>
	<xsl:attribute name = "y">
	 <xsl:value-of select = "$text_y + ($text_indent + $font_h)" />
	</xsl:attribute>

	<xsl:value-of select = "@arrival" />
       </text>

       <text>
	<xsl:attribute name = "x">
	 <xsl:value-of select = "$text_x + $text_indent" />
	</xsl:attribute>
	<xsl:attribute name = "y">
	 <xsl:value-of select = "$text_y + (($text_indent + $font_h) * 2)" />
	</xsl:attribute>
	<xsl:value-of select = "name" />
       </text>

      </g>

     </xsl:template>

<!-- ====================================================================== 

     This is such an annoying and unpleasant hack...
     There doesn't appear to be any way to get the current node's sibling
     in a sort context - that is (a,b,c) becomes (b,a,c) and the sibling of
     (a) is always (b)
     ====================================================================== -->

     <xsl:template name = "goto-next">
      <xsl:param name = "cx" />
      <xsl:param name = "cy" />
      <xsl:param name = "arrival" />
      <xsl:param name = "now" />

       <xsl:variable name = "name"    select = "name" />
       <xsl:variable name = "arrival" select = "@arrival" />
       <xsl:variable name = "pos"     select = "position()" />

       <xsl:for-each select = "/travels/location[date:seconds(@arrival) > date:seconds($arrival)]">
        <xsl:sort select    = "date:seconds(@arrival)"
		  data-type = "number" />

        <xsl:call-template name = "goto">
	 <xsl:with-param name = "cx"   select = "$cx" />
	 <xsl:with-param name = "cy"   select = "$cy" />
	 <xsl:with-param name = "from" select = "$name" />
	 <xsl:with-param name = "now"  select = "$now" />
	 <xsl:with-param name = "pos"  select = "$pos" />
	 <xsl:with-param name = "inst" select = "position()" />
	</xsl:call-template>
       </xsl:for-each>
     </xsl:template>
 
<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "calc-x">
      <xsl:param name = "long" />

      <xsl:variable name = "l">
      <xsl:call-template name = "prune-long">
       <xsl:with-param name = "long" select = "$long"/>
      </xsl:call-template>
      </xsl:variable>

      <xsl:value-of select = "(round(($l - $map_min_x) * $map_scale * $map_exp) div $map_exp)" />
     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "prune-long">
      <xsl:param name = "long" />

      <xsl:variable name = "len" select = "string-length($long)" />

      <xsl:choose>
       <xsl:when test = "substring($long,$len,1) = 'W'">
        <xsl:text>-</xsl:text>
        <xsl:value-of select = "substring($long,1,($len - 1))" />
       </xsl:when>
       <xsl:when test = "substring($long,$len,1) = 'E'">
        <xsl:value-of select = "substring($long,1,($len - 1))" />
       </xsl:when>
       <xsl:otherwise>
        <xsl:value-of select = "$long" />
       </xsl:otherwise>
      </xsl:choose>

     </xsl:template>
      
<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "calc-y">
      <xsl:param name = "lat" />

      <xsl:variable name = "l">
      <xsl:call-template name = "prune-lat">
       <xsl:with-param name = "lat" select = "$lat"/>
      </xsl:call-template>
      </xsl:variable>

      <xsl:value-of select = "(round(($map_max_y - $l) * $map_scale * $map_exp) div $map_exp)" />
     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "prune-lat">
      <xsl:param name = "lat" />

      <xsl:variable name = "len" select = "string-length($lat)" />

      <xsl:choose>
       <xsl:when test = "substring($lat,$len,1) = 'N'">
        <xsl:value-of select = "substring($lat,1,($len - 1))" />
       </xsl:when>
       <xsl:when test = "substring($lat,$len,1) = 'S'">
        <xsl:text>-</xsl:text>
        <xsl:value-of select = "substring($lat,1,($len - 1))" />
       </xsl:when>
       <xsl:otherwise>
        <xsl:value-of select = "$lat" />
       </xsl:otherwise>
      </xsl:choose>

     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "goto">
      <xsl:param name = "cx" />
      <xsl:param name = "cy" />
      <xsl:param name = "from" />
      <xsl:param name = "now" />
      <xsl:param name = "pos" />
      <xsl:param name = "inst" />

      <!-- see notes above -->

      <xsl:if test = "$inst = 1">

        <xsl:comment>
	 <xsl:value-of select = "$from" />
	 <xsl:text> to </xsl:text>
	 <xsl:value-of select = "name" />
	</xsl:comment>

        <xsl:variable name = "secs_then">
         <xsl:value-of select = "date:seconds(@arrival)" />
        </xsl:variable>

        <xsl:variable name = "plot_when">
         <xsl:choose>
	  <xsl:when test = "$secs_then > $now">
	   <xsl:text>future</xsl:text>
	  </xsl:when>
	  <xsl:otherwise>
	   <xsl:text>past</xsl:text>
	  </xsl:otherwise>
	 </xsl:choose>
        </xsl:variable>

       <xsl:variable name = "dx">
       <xsl:call-template name = "calc-x">
        <xsl:with-param name = "long">
	 <xsl:value-of select = "longitude" />
        </xsl:with-param>
       </xsl:call-template>
       </xsl:variable>

       <xsl:variable name = "dy">
       <xsl:call-template name = "calc-y">
        <xsl:with-param name = "lat">
	 <xsl:value-of select = "latitude" />
        </xsl:with-param>
       </xsl:call-template>
       </xsl:variable>

       <xsl:choose>
	<xsl:when test = "@transpacific and $cx > $dx">
	 <xsl:call-template name = "goto-pac-we">
          <xsl:with-param name = "cx" select = "$cx" />
          <xsl:with-param name = "cy" select = "$cy" />
          <xsl:with-param name = "dx" select = "$dx" />
          <xsl:with-param name = "dy" select = "$dy" />
	  <xsl:with-param name = "pos" select       = "$pos" />
	  <xsl:with-param name = "plot_when" select = "$plot_when" />
	 </xsl:call-template>
	</xsl:when>
	<xsl:when test = "@transpacific">
	 <xsl:call-template name = "goto-pac-ew">
          <xsl:with-param name = "cx" select = "$cx" />
          <xsl:with-param name = "cy" select = "$cy" />
          <xsl:with-param name = "dx" select = "$dx" />
          <xsl:with-param name = "dy" select = "$dy" />
	  <xsl:with-param name = "pos" select = "$pos" />
	  <xsl:with-param name = "plot_when" select = "$plot_when" />
	 </xsl:call-template>
	</xsl:when>
	<xsl:otherwise>
       <line>
        <xsl:attribute name = "class">
	 <xsl:text>goto-</xsl:text>
	 <xsl:value-of select = "$plot_when" />
	</xsl:attribute>

        <xsl:attribute name = "x1">
	 <xsl:value-of select = "$cx" />
	</xsl:attribute>

        <xsl:attribute name = "y1">
	 <xsl:value-of select = "$cy" />
	</xsl:attribute>

        <xsl:attribute name = "x2">
	 <xsl:value-of select = "$cx" />
	</xsl:attribute>

        <xsl:attribute name = "y2">
	 <xsl:value-of select = "$cy" />
	</xsl:attribute>

	<animate>
         <xsl:attribute name = "attributeName">x2</xsl:attribute>
         <xsl:attribute name = "attributeType">XML</xsl:attribute>

	 <xsl:attribute name = "from">
	  <xsl:value-of select = "$cx" />
	 </xsl:attribute>

	 <xsl:attribute name = "to">
	  <xsl:value-of select = "$dx" />
	 </xsl:attribute>

	 <xsl:attribute name = "begin">
	  <xsl:value-of select = "($pos - 1) * $travel_duration" />
	  <xsl:text>s</xsl:text>
         </xsl:attribute>
	 <xsl:attribute name = "dur">
	  <xsl:value-of select = "$travel_duration" />
	  <xsl:text>s</xsl:text>
	 </xsl:attribute>
	 <xsl:attribute name = "fill">freeze</xsl:attribute>
	</animate>

	<animate>
         <xsl:attribute name = "attributeName">y2</xsl:attribute>
         <xsl:attribute name = "attributeType">XML</xsl:attribute>

	 <xsl:attribute name = "from">
	  <xsl:value-of select = "$cy" />
	 </xsl:attribute>

 	 <xsl:attribute name = "to">
	  <xsl:value-of select = "$dy" />
	 </xsl:attribute>

	 <xsl:attribute name = "begin">
	  <xsl:value-of select = "($pos - 1) * $travel_duration" />
	  <xsl:text>s</xsl:text>
         </xsl:attribute>
	 <xsl:attribute name = "dur">
	  <xsl:value-of select = "$travel_duration" />
	  <xsl:text>s</xsl:text>
	 </xsl:attribute>
	 <xsl:attribute name = "fill">freeze</xsl:attribute>
	</animate>

        </line>
	</xsl:otherwise>
	</xsl:choose>

      </xsl:if>

     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "goto-pac-we">
       <xsl:param name = "cx" />
       <xsl:param name = "cy" />
       <xsl:param name = "dx" />
       <xsl:param name = "dy" />

       <xsl:param name = "pos" />
       <xsl:param name = "plot_when" />

      <xsl:variable name = "a">
       <xsl:value-of select = "$cx" />
      </xsl:variable>

      <xsl:variable name = "b">
       <xsl:value-of select = "$a + ($map_w - $dx)" />
      </xsl:variable>

      <xsl:variable name = "c"  select = "$cy - $dy" />
      <xsl:variable name = "d"  select = "$b div $a" />
      <xsl:variable name = "dd" select = "$c div $d" />
      <xsl:variable name = "e"  select = "round($cy - $dd)" />

      <xsl:call-template name = "goto-pac">
       <xsl:with-param name = "cx" select = "$cx" />
       <xsl:with-param name = "cy" select = "$cy" />
       <xsl:with-param name = "dx" select = "$dx" />
       <xsl:with-param name = "dy" select = "$dy" />

       <xsl:with-param name = "pos" select = "$pos" />
       <xsl:with-param name = "plot_when" select = "$plot_when" />

       <xsl:with-param name = "a" select = "$a" />
       <xsl:with-param name = "b" select = "$b" />
       <xsl:with-param name = "c" select = "$c" />
       <xsl:with-param name = "d" select = "$d" />
       <xsl:with-param name = "dd" select = "$dd" />
       <xsl:with-param name = "e" select = "$e" />
       <xsl:with-param name = "f" select = "$map_w" />
       <xsl:with-param name = "g" select = "0" />
      </xsl:call-template>

     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "goto-pac-ew">
       <xsl:param name = "cx" />
       <xsl:param name = "cy" />
       <xsl:param name = "dx" />
       <xsl:param name = "dy" />

       <xsl:param name = "pos" />
       <xsl:param name = "plot_when" />

       <xsl:variable name = "a">
	<xsl:choose>
	 <xsl:when test = "$dy > $cy">
	  <xsl:value-of select = "$cx" />
	 </xsl:when>
	 <xsl:otherwise>
          <xsl:value-of select = "$map_w - $cx" />
	 </xsl:otherwise>
	</xsl:choose>
       </xsl:variable>

       <xsl:variable name = "b">
	<xsl:value-of select = "$a + $dx" />
       </xsl:variable>

       <xsl:variable name = "c">
	<xsl:choose>
	 <xsl:when test = "$dy > $cy">
 	  <xsl:value-of select = "$dy - $cy" />
	 </xsl:when>
	 <xsl:otherwise>
 	  <xsl:value-of select = "$cy - $dy" />
	 </xsl:otherwise>
	</xsl:choose>
       </xsl:variable>

       <xsl:variable name = "d"  select = "$b div $a" />

       <xsl:variable name = "dd" select = "$c div $d" />

       <xsl:variable name = "e">
	<xsl:choose>
	 <xsl:when test = "$dy > $cy">
	  <xsl:value-of select = "$cy + $dd" />
	 </xsl:when>
	 <xsl:otherwise>
 	  <xsl:value-of select = "round($cy - $dd)" />
	 </xsl:otherwise>
	</xsl:choose>
       </xsl:variable>

      <xsl:call-template name = "goto-pac">
       <xsl:with-param name = "cx" select = "$cx" />
       <xsl:with-param name = "cy" select = "$cy" />
       <xsl:with-param name = "dx" select = "$dx" />
       <xsl:with-param name = "dy" select = "$dy" />

       <xsl:with-param name = "pos" select = "$pos" />
       <xsl:with-param name = "plot_when" select = "$plot_when" />

       <xsl:with-param name = "a" select = "$a" />
       <xsl:with-param name = "b" select = "$b" />
       <xsl:with-param name = "c" select = "$c" />
       <xsl:with-param name = "d" select = "$d" />
       <xsl:with-param name = "dd" select = "$dd" />
       <xsl:with-param name = "e" select = "$e" />
       <xsl:with-param name = "f" select = "0" />
       <xsl:with-param name = "g" select = "$map_w" />
      </xsl:call-template>

     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "goto-pac">
      <xsl:param name = "cx" />
      <xsl:param name = "cy" />
      <xsl:param name = "dx" />
      <xsl:param name = "dy" />

      <xsl:param name = "pos" />
      <xsl:param name = "plot_when" />

      <xsl:param name = "a" />
      <xsl:param name = "b" />
      <xsl:param name = "c" />
      <xsl:param name = "d" />
      <xsl:param name = "dd" />
      <xsl:param name = "e" />
      <xsl:param name = "f" />

      <!--
	 <xsl:comment>
	  a:<xsl:value-of select  = "$a" />
	  cx:<xsl:value-of select = "$cx" />
	  b:<xsl:value-of select  = "$b" />
	  cy:<xsl:value-of select = "$cy" />
	  dy:<xsl:value-of select = "$dy" />
	  c:<xsl:value-of select  = "$c" />
	  d:<xsl:value-of select  = "$d" />
	  dd:<xsl:value-of select = "$dd" />
	  e:<xsl:value-of select  = "$e" />
	  f:<xsl:value-of select  = "$f" />
	  g:<xsl:value-of select  = "$g" />
	 </xsl:comment>
       -->

       <!-- store travel_duration / 2 as a variable since
	    we need to use it below setting times for the 
	    two lines -->

       <xsl:variable name = "half_dur">
	<xsl:value-of select = "($travel_duration div 2)" />
       </xsl:variable>

       <!-- first line, to edge -->

       <line>
        <xsl:attribute name = "class">
	 <xsl:text>goto-</xsl:text>
	 <xsl:value-of select = "$plot_when" />
	</xsl:attribute>

        <xsl:attribute name = "x1">
	 <xsl:value-of select = "$cx" />
	</xsl:attribute>

        <xsl:attribute name = "y1">
	 <xsl:value-of select = "$cy" />
	</xsl:attribute>

        <xsl:attribute name = "x2">
	 <xsl:value-of select = "$cx" />
	</xsl:attribute>

        <xsl:attribute name = "y2">
	 <xsl:value-of select = "$cy" />
	</xsl:attribute>

 	<animate>
         <xsl:attribute name = "attributeName">x2</xsl:attribute>
         <xsl:attribute name = "attributeType">XML</xsl:attribute>

	 <xsl:attribute name = "from">
	  <xsl:value-of select = "$cx" />
	 </xsl:attribute>

	 <xsl:attribute name = "to">
	  <xsl:value-of select = "$f" />
	 </xsl:attribute>

	 <xsl:attribute name = "begin">
	  <xsl:value-of select = "($pos - 1) * $travel_duration" />
	  <xsl:text>s</xsl:text>
         </xsl:attribute>

	 <xsl:attribute name = "dur">
	  <xsl:value-of select = "$half_dur" />
	  <xsl:text>s</xsl:text>
	 </xsl:attribute>

	 <xsl:attribute name = "fill">freeze</xsl:attribute>
	</animate>

 	<animate>
         <xsl:attribute name = "attributeName">y2</xsl:attribute>
         <xsl:attribute name = "attributeType">XML</xsl:attribute>

	 <xsl:attribute name = "from">
	  <xsl:value-of select = "$cy" />
	 </xsl:attribute>

	 <xsl:attribute name = "to">
	  <xsl:value-of select = "$e" />
	 </xsl:attribute>

	 <xsl:attribute name = "begin">
	  <xsl:value-of select = "($pos - 1) * $travel_duration" />
	  <xsl:text>s</xsl:text>
         </xsl:attribute>

	 <xsl:attribute name = "dur">
	  <xsl:value-of select = "$half_dur" />
	  <xsl:text>s</xsl:text>
	 </xsl:attribute>

	 <xsl:attribute name = "fill">freeze</xsl:attribute>
	</animate>
       </line>

      <!-- second line, from edge -->

      <line>
        <xsl:attribute name = "class">
	 <xsl:text>goto-</xsl:text>
	 <xsl:value-of select = "$plot_when" />
	</xsl:attribute>

        <xsl:attribute name = "x1">
	 <xsl:value-of select = "$g" />
	</xsl:attribute>

        <xsl:attribute name = "y1">
	 <xsl:value-of select = "$e" />
	</xsl:attribute>

        <xsl:attribute name = "x2">
	 <xsl:value-of select = "$g" />
	</xsl:attribute>

        <xsl:attribute name = "y2">
	 <xsl:value-of select = "$e" />
	</xsl:attribute>

	<animate>
         <xsl:attribute name = "attributeName">x2</xsl:attribute>
         <xsl:attribute name = "attributeType">XML</xsl:attribute>

	 <xsl:attribute name = "from">
	  <xsl:value-of select = "$g" />
	 </xsl:attribute>

 	 <xsl:attribute name = "to">
	  <xsl:value-of select = "$dx" />
	 </xsl:attribute>

	 <xsl:attribute name = "begin">
	  <xsl:value-of select = "(($pos - 1) * $travel_duration) + $half_dur" />
	  <xsl:text>s</xsl:text>
         </xsl:attribute>

	 <xsl:attribute name = "dur">
	  <xsl:value-of select = "$half_dur" />
	  <xsl:text>s</xsl:text>
	 </xsl:attribute>

	 <xsl:attribute name = "fill">freeze</xsl:attribute>
	</animate>

	<animate>
         <xsl:attribute name = "attributeName">y2</xsl:attribute>
         <xsl:attribute name = "attributeType">XML</xsl:attribute>

	 <xsl:attribute name = "from">
	  <xsl:value-of select = "$e" />
	 </xsl:attribute>

 	 <xsl:attribute name = "to">
	  <xsl:value-of select = "$dy" />
	 </xsl:attribute>

	 <xsl:attribute name = "begin">
	  <xsl:value-of select = "(($pos - 1) * $travel_duration) + $half_dur" />
	  <xsl:text>s</xsl:text>
         </xsl:attribute>

	 <xsl:attribute name = "dur">
	  <xsl:value-of select = "$half_dur" />
	  <xsl:text>s</xsl:text>
	 </xsl:attribute>

	 <xsl:attribute name = "fill">freeze</xsl:attribute>
	</animate>

	</line>
     </xsl:template>

<!-- ====================================================================== 

     ====================================================================== -->

     <xsl:template name = "you-are-here">
      <xsl:param name = "lat" />
      <xsl:param name = "long" />
      <xsl:param name = "inst" />

      <!-- see notes above -->

      <xsl:if test = "$inst = 1">

       <xsl:variable name = "cx">
       <xsl:call-template name = "calc-x">
        <xsl:with-param name = "long">
	 <xsl:value-of select = "longitude" />
        </xsl:with-param>
       </xsl:call-template>
       </xsl:variable>

       <xsl:variable name = "cy">
        <xsl:call-template name = "calc-y">
         <xsl:with-param name = "lat">
	  <xsl:value-of select = "latitude" />
         </xsl:with-param>
        </xsl:call-template>
       </xsl:variable>

      <xsl:variable name = "point_from_below">
       <xsl:if test = "$cy &lt;= 30">
        <xsl:text>1</xsl:text>
       </xsl:if>
      </xsl:variable>

      <xsl:variable name = "point_when">
       <xsl:value-of select = "(count(/travels/location) - 1) * $travel_duration" />
      </xsl:variable>

      <use>

       <xsl:attribute name = "xlink:href">
        <xsl:text>#you-are-here</xsl:text>
       </xsl:attribute>

       <xsl:attribute name = "x">
        <xsl:value-of select = "$cx - 15" />
       </xsl:attribute>

       <xsl:attribute name = "y">
        <xsl:value-of select = "-30" />
       </xsl:attribute>

       <xsl:if test = "$point_from_below = 1">
       <xsl:attribute name = "transform">
        <xsl:text>translate(0,</xsl:text>
	<xsl:value-of select = "$map_h + 20" />
        <xsl:text>) scale(1,-1)</xsl:text>
       </xsl:attribute>
       </xsl:if>

       <animate>
        <xsl:attribute name = "attributeType">XML</xsl:attribute>
        <xsl:attribute name = "attributeName">y</xsl:attribute>
        <xsl:attribute name = "from">-30</xsl:attribute>

        <xsl:attribute name = "to">
	 <xsl:choose>
	  <xsl:when test = "$point_from_below = 1">
  	   <xsl:value-of select = "$map_h - $cy" />
	  </xsl:when>
	  <xsl:otherwise>
	   <xsl:value-of select = "$cy - 20" />
	  </xsl:otherwise>
	 </xsl:choose>
	</xsl:attribute>

	<xsl:attribute name = "begin">
	 <xsl:value-of select = "$point_when" />
	 <xsl:text>s</xsl:text>
	</xsl:attribute>

	<xsl:attribute name = "dur">
	 <xsl:text>2.5s</xsl:text>
	</xsl:attribute>

	<xsl:attribute name = "fill">
	 <xsl:text>freeze</xsl:text>
	</xsl:attribute>

       </animate>

       <animate>
        <xsl:attribute name = "attributeType">XML</xsl:attribute>
        <xsl:attribute name = "attributeName">y</xsl:attribute>

        <xsl:attribute name = "from">
	 <xsl:choose>
	  <xsl:when test = "$point_from_below = 1">
  	   <xsl:value-of select = "$map_h - $cy" />
	  </xsl:when>
	  <xsl:otherwise>
	   <xsl:value-of select = "$cy - 20" />
	  </xsl:otherwise>
	 </xsl:choose>
	</xsl:attribute>

        <xsl:attribute name = "to">
	 <xsl:choose>
	  <xsl:when test = "$point_from_below = 1">
  	   <xsl:value-of select = "$map_h - ($cy + 10)" />
	  </xsl:when>
	  <xsl:otherwise>
	   <xsl:value-of select = "$cy - 30" />
	  </xsl:otherwise>
	 </xsl:choose>
	</xsl:attribute>

	<xsl:attribute name = "begin">
	 <xsl:value-of select = "$point_when + 2.5" />
	 <xsl:text>s</xsl:text>
	</xsl:attribute>

	<xsl:attribute name = "dur">
	 <xsl:text>0.5s</xsl:text>
	</xsl:attribute>

	<xsl:attribute name = "fill">
	 <xsl:text>freeze</xsl:text>
	</xsl:attribute>

       </animate>

      </use>

      </xsl:if>
     </xsl:template>

<!-- ====================================================================== 

     This is normally found in the XSLTSL libraries but the last time
     I checked the template didn't actually work as advertised and I
     still haven't gotten around to submitting a patch...
     ====================================================================== -->

     <xsl:template name='markup:cdata-section'>
      <xsl:param name='text'/>

      <xsl:if test="contains($text, ']]&gt;')">
       <xsl:message terminate="yes">CDATA section contains "]]&gt;"</xsl:message>
      </xsl:if>

      <xsl:text disable-output-escaping='yes'>&lt;![CDATA[</xsl:text>
      <xsl:copy-of select='$text'/>

      <!-- sigh, the newline after &gt;
	   is important... -->

      <xsl:text disable-output-escaping='yes'>// ]]&gt;
</xsl:text>
     </xsl:template>

<!-- ====================================================================== 
     FIN // $Id: svg-travel-map.xsl,v 1.18 2004/09/15 05:19:53 asc Exp $
     ====================================================================== -->

</xsl:stylesheet>
